/***************************************************************************\
  "juli.c" - Juliet Interpreter Controller Program
  (see below for version and date)
  Copyright (C) 2002  Kevin P. Albrecht
\***************************************************************************/

/////////////////////////
// DEFINES //////////////
/////////////////////////

// Environment //
#define ENV_GCC
//#define ENV_LCC
//#define ENV_MSVC

// Debug Status //
//#define GLOBAL_DEBUG
#define GLOBAL_NODEBUG

// Miscellaneous //
#define GLOBAL_MANTISSA_SIZE 5
#define GLOBAL_TOKEN_SIZE 256
#define GLOBAL_MAX_SCOPES 256
#define GLOBAL_MAX_SYMBOLS 16384

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //
#include <stdio.h>
#include <stdlib.h>

// Local - Pre Global Variable Declaration //
#include "global.h"
#include "error.h"
#include "table.h"
#include "scope.h"
#include "token.h"

// Local - Post Global Variable Declaration //
#include "inputbuffer.h"
#include "lexer.h"
#include "parser.h"
#include "tempgen.h"
#include "k_string.h"
#include "global_conv.h"
#include "re.h"
#include "icoutput.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

int main (int nArgs, char *vsArgs[])
{
  // Vars - Declare //
  char *filenameInput;
  char *filenameOutput;
  int inArg;
  bool bOutput; // true = output Romeo Code; false = run program

  // Vars - Define //
  filenameInput = k_new (256); k_copy (filenameInput, "I AM EMPTY!!");
  filenameOutput = k_new (256); k_copy (filenameOutput, "I AM EMPTY!!");
  bOutput = false;

  // Initialize Global //
  global_init ();

  // Command-line Argument Handling //
  if (nArgs > 1)
  {
    for (inArg = 1; inArg < nArgs; inArg++)
    {
      // '-a', '--about' //
      if ((k_same (vsArgs[inArg], "-a")) || (k_same (vsArgs[inArg], "--about")))
      {
        puts ("");
        puts ("========================= Juliet Interpreter (JULI) =========================");
        puts ("");
        puts ("Notice:  Copyright (C) 2002, Kevin P. Albrecht");
        puts ("E-mail:  kevin@albrecht.net");
        puts ("Website: http://kevin.alteu.com/");
        puts ("");
        puts ("JULI Version: 0.03.0141 (22 Nov 2002)");
        puts ("Juliet Spec:  0.03 (22 Nov 2002)");
        puts ("Romeo Spec:   0.09 (22 Nov 2002)");
        return 0;
      }
      // '-l', '--license' //
      if ((k_same (vsArgs[inArg], "-l")) || (k_same (vsArgs[inArg], "--license")))
      {
        puts ("");
        puts ("Copyright (C) 2000-2002, Kevin P. Albrecht");
        puts ("");
        puts ("Permission is hereby granted by Kevin P. Albrecht (the \"Author\"), free of");
        puts ("charge, to any person obtaining a copy of this software and associated");
        puts ("documentation files (the \"Software\"), to deal in the Software without");
        puts ("restriction, including without limitation the rights to use, copy, modify,");
        puts ("merge, publish, distribute, sublicense, and/or sell copies of the");
        puts ("Software, and to permit persons to whom the Software is furnished to do");
        puts ("so, subject to the following conditions:");
        puts ("The above copyright notice and this permission notice shall be included");
        puts ("in all copies or substantial portions of the Software.");
        puts ("");
        puts ("THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR");
        puts ("IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,");
        puts ("FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL");
        puts ("THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN");
        puts ("AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN");
        puts ("CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.");
        puts ("");
        puts ("Except as contained in this notice, the name of the Author shall not be");
        puts ("used in advertising or otherwise to promote the sale, use or other");
        puts ("dealings in this Software without prior written authorization from the");
        puts ("Author.");
        return 0;
      }
      // '-v', '--verbose' //
      else if ((k_same (vsArgs[inArg], "-v")) || (k_same (vsArgs[inArg], "--verbose")))
      {
        gbVerbose = true;
      }
      // '-i' //
      else if (k_same (vsArgs[inArg], "-i"))
      {
        if ((inArg+1) < nArgs)
        {
          bOutput = true;
          k_copy (filenameOutput, vsArgs[++inArg]);
        }
        else
        {
          err (false, EP_MISSING_ARG, "output filename");
          erre (EFF_TERMINATE);
        }
      }
      else
      {
        if (vsArgs[inArg][0] == '-')
        {
          err (false, EP_INVALID_ARG, vsArgs[inArg]);
          erre (EFF_TERMINATE);
        }
        else
        {
          k_copy (filenameInput, vsArgs[inArg]);
          break;
        }
      }
    }
  }
  else
  {
    puts ("Usage: juli [ option(s) ] [ input file ]");
    puts ("");
    puts ("Option           | Description                                      ");
    puts ("-----------------+--------------------------------------------------");
    puts ("-a, --about      | display release and contact info                 ");
    puts ("-l, --license    | display distribution license                     ");
    puts ("-v, --verbose    | print verbose information                        ");
    puts ("-i X             | output Romeo file called X; do not interpret     ");
    return 0;
  }

  // Verify the capture of the filename //
  if (k_same (filenameInput, "I AM EMPTY!!"))
  {
    err (false, EP_MISSING_ARG, "input filename");
    erre (EFF_TERMINATE);
  }

  // Initialize Phase A: Input //
  ib_init ();
  ib_include (filenameInput);

  // Initialize Phase B: Lexical Analysis //
  lex_init ();

  // Initialize Phase C: Syntax Analysis //
  parser_init ();
  scope_init ();

  // Initialize Phase D: Intermediate Code //
  icm_init ();
  tempgen_init ();

  // Begin processing //
  parser_parse(); // parse the program
  if (gnErrors <= 0) // continue if no errors reported in parsing
  {
    icm_reset(); // reset the intermediate code position marker
    if (bOutput)
    {
      icoutput (filenameOutput); // output intermediate code in Romeo Language
    }
    else
    {
      re_init ();
      re_run ();
      re_deinit ();
    }
  }

  // Deinitialize Phase D: Intermediate Code //
  tempgen_deinit ();
  icm_deinit ();

  // Deinitialize Phase C: Syntax Analysis //
  scope_deinit ();
  parser_deinit ();

  // Deinitialize Phase B: Lexical Analysis //
  lex_deinit ();

  // Deinitialize Phase A: Input //
  ib_deinit ();

  // Deinitialize Global //
  global_deinit ();

  // Vars - Free //
  k_del (filenameInput);
  k_del (filenameOutput);

  return 0;
}
